<?php

namespace FiloBlu\AdyenErrorNotificationHandler\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;

class Data
{
    const ADYEN_ERROR_NOTIFICATION_HANDLER_ENABLED_CONFIG_PATH = 'adyen_error_notification_handler/general/error_notification_enabled';
    const ADYEN_ERROR_NOTIFICATION_HANDLER_MESSAGES_CONFIG_PATH = 'adyen_error_notification_handler/general/error_notification_messages';
    const ADYEN_ERROR_NOTIFICATION_HANDLER_GENERAL_USE_DEFAULT_LANG = 'adyen_error_notification_handler/general/use_default_lang';
    const ADYEN_ERROR_NOTIFICATION_HANDLER_LOG_ENABLED_CONFIG_PATH = 'adyen_error_notification_handler/general/enable_log';
    const ADYEN_REFUSAL_REASON_CODE = 'refusalReasonCode';
    const ERROR_CODE_3DV1 = '3dv1';
    const DEFAULT_STORE = 'default';
    const LOG_FILE_NAME = '/var/log/adyen-error-notification-handler.log';

    /**
     * @var ScopeConfigInterface
     */
    protected $config;

    private $logger = [];

    public function __construct(
        ScopeConfigInterface $config
    ) {
        $this->config = $config;
    }

    public function log($toLog)
    {
        if($this->logIsEnabled()) {
            $loggerData = $this->getLogger();
            $logger = $loggerData['logger'];
            $logger->info($toLog);
        }
    }

    public function logIsEnabled()
    {
        return $this->config->isSetFlag(self::ADYEN_ERROR_NOTIFICATION_HANDLER_LOG_ENABLED_CONFIG_PATH);
    }

    private function getLogger()
    {
        if(isset($this->logger['logger'])) {
            return $this->logger;
        }

        $writer = new \Zend\Log\Writer\Stream(BP . self::LOG_FILE_NAME);
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $this->logger['logger'] = $logger;

        return $this->logger;
    }

    public function adyenErrorNotificationHandlerIsEnabled()
    {
        return $this->config->isSetFlag(self::ADYEN_ERROR_NOTIFICATION_HANDLER_ENABLED_CONFIG_PATH);
    }

    public function getAdyenErrorNotificationMessages()
    {
        return $this->config->getValue(self::ADYEN_ERROR_NOTIFICATION_HANDLER_MESSAGES_CONFIG_PATH);
    }

    public function useErrorMessagesInDefaultLangEnabled()
    {
        return $this->config->isSetFlag(self::ADYEN_ERROR_NOTIFICATION_HANDLER_GENERAL_USE_DEFAULT_LANG);
    }

    public function getErrorsInLanguage($errorsList, $languageIso)
    {
        if(isset($errorsList[$languageIso]))
        {
            return $errorsList[$languageIso];
        }

        $language = explode('_', $languageIso);

        foreach ($language as $lang)
        {
            if(isset($errorsList[$lang]))
            {
                return $errorsList[$lang];
            }
        }

        return null;
    }

    public function getErrorByCode($errorsList, $errorCode)
    {
        if(isset($errorsList[$errorCode]))
        {
            return $errorsList[$errorCode];
        }

        return null;
    }

    public function getDefaultLang()
    {
        return $this->config->getValue(
            \Magento\Directory\Helper\Data::XML_PATH_DEFAULT_LOCALE,
            self::DEFAULT_STORE
        );
    }

    public function manageDataToLog($data)
    {
        $this->log('*********');
        $this->log('User language: '.$data['lang']);

        if($data['3ds_v1'])
        {
            $this->log('3d secure V1 transaction');
        }

        if(isset($data['response']['pspReference']))
        {
            $this->log('Adyen pspReference: '.$data['response']['pspReference']);
        }
        if(isset($data['response']['refusalReason']))
        {
            $this->log('Adyen refusalReason: '.$data['response']['refusalReason']);
        }
        if(isset($data['response']['resultCode']))
        {
            $this->log('Adyen resultCode: '.$data['response']['resultCode']);
        }
        if(isset($data['response']['refusalReasonCode']))
        {
            $this->log('Adyen refusalReasonCode: '.$data['response']['refusalReasonCode']);
        }
        else
        {
            $this->log("Adyen doesn't send refusalReasonCode!");
        }
        if(isset($data['response']['merchantReference']))
        {
            $this->log('Merchant Reference: '.$data['response']['merchantReference']);
        }

        if(isset($data['module_error']))
        {
            $this->log('mod2-adyen-error-notification-handler returned the error: '.$data['module_error']);
        }
    }
}